/*
  SD card datalogger
  Hardware: Olimexino 328 (Arduino Nano Klon) mit Eibnstellung auf 3,3 Volt
            oder: Arduino UNO mit SD-Card-Shield, z.B. Ethernet-Shield

 
 The circuit:
 * analog sensors on analog inputs A0 ... A5 (selectable)
 * SD card attached to SPI bus as follows (UNO, Olimexino 328)
 * 
 ** SD-Pin          -> Arduino-Pin
 ** ---------------------------------------------
 ** Pin 1 CD/DATA3  -> CS -> Pin 4 (Chip Select)
 ** Pin 2 CMD       -> MOSI Pin 11
 ** Pin 3 GND       -> GND
 ** Pin 4 3,3Volt   -> Ext. Spgversorgung 3,3 Volt(!)
 ** Pin 5 CLK       -> CLK -> Pin 13
 ** Pin 6 GND       -> GND (optional)
 ** Pin 7 DATA0     -> MISO -> Pin 12
 
  This example code is in the public domain.


   Commands:
      EEPROM-Adresse 0        : A1..6;     - Analog Channel used, Example: A1;
      EEPROM-Adresse 2+3+4+5  : D[ms];     - Delay in ms, Example D10000;
      
 */

#include <EEPROM.h>
#include <SPI.h>
#include <SD.h>

// SD-Card
const int chipSelect = 4;
unsigned long count = 0;


// EEPROM
// Benutzte analoge Kanäle auf EEPROM-Adsresse 0
// Messintervall auf EEPRO;-Adresse ab 2

// Zeit
unsigned long last;                                       // Zeit des letzten Schreibens auf SD-Card

// UART
int16_t wert[] =  {0, 0, 0, 0, 0};                        // per Funk übermittelte Werte
char  uart_inputstr [20]  = "0";
int i = 0;
boolean complete = false;                                 // wird true, wenn ein String komplett ist

// benutzte analoge Kanäle und Zeit zwischen den Messungen
uint16_t analoge_kanaele;                                 // benutzte analoge Eingänge
uint16_t rohwert = 0;                                     // Rohwert eines analogen Kanals
float messintervall_f;                                    // Messintervall als float
unsigned long messintervall;                              // dasselbe aber ganzzahlig
bool messung = true;                                      // Soll eine Messung erfolgen
uint8_t enable_analog_0 = 0;
uint8_t enable_analog_1 = 0;
uint8_t enable_analog_2 = 0;
uint8_t enable_analog_3 = 0;
uint8_t enable_analog_4 = 0;
uint8_t enable_analog_5 = 0;
uint8_t channels_count = 0;                               // Anzahl der benutzten analogen Kanäle

//split_int( int sixteen_bit)
//{
//  high_byte = (int)(sixteen_bit >> 8)); // High Byte
//  low_byte  = ((int)(sixteen_bit & 0xFF)); // Low Byte
//
//  unsigned long l = 100000;                 //Variable to store in EEPROM.
//  int eeAddress = 0;                        //Location we want the data to be put.
//  //One simple call, with the address first and the object second.
//  EEPROM.put(eeAddress, l);
//}

void eeprom_lesen()
{
  EEPROM.get(0, analoge_kanaele);
  Serial.print("EEPROM analoge Kanäle (binaer): "); Serial.println(analoge_kanaele, BIN);
  channels_count = 0;
  if(bitRead(analoge_kanaele, 0)) { enable_analog_0 = 1; channels_count++; } else { enable_analog_0 = 0; }
  if(bitRead(analoge_kanaele, 1)) { enable_analog_1 = 1; channels_count++; } else { enable_analog_1 = 0; }
  if(bitRead(analoge_kanaele, 2)) { enable_analog_2 = 1; channels_count++; } else { enable_analog_2 = 0; }
  if(bitRead(analoge_kanaele, 3)) { enable_analog_3 = 1; channels_count++; } else { enable_analog_3 = 0; }
  if(bitRead(analoge_kanaele, 4)) { enable_analog_4 = 1; channels_count++; } else { enable_analog_4 = 0; }
  if(bitRead(analoge_kanaele, 5)) { enable_analog_5 = 1; channels_count++; } else { enable_analog_5 = 0; }    

  EEPROM.get(2, messintervall_f);
  messintervall = (unsigned long) messintervall_f;
  Serial.print("EEPROM messintervall: "); Serial.println(messintervall);
}

void uart_auswertung()
{
  char tempstr [10] = "012346789";
  // tempstr löschen
  for(int i; i< sizeof(tempstr) ; i++) { tempstr[i] = 0; }
  
  if(uart_inputstr[0] == 'A' || uart_inputstr[0] == 'D')  // Analo Channels oder Delay?
  {
    uint8_t i = 1;    // Kopieren in tempstr ohne A oder D
    while(uart_inputstr[i] != ';')
    {
      tempstr[i-1] = uart_inputstr[i];
      i++;
    }
    // In tempstr steht nun der Wert in ASCII

    // Befehl zu den Analogen Kanälen verarbeiten
    if(uart_inputstr[0] == 'A') 
    { 
      analoge_kanaele = atoi(tempstr); 
      EEPROM.put(0, analoge_kanaele);
      channels_count = 0;
      if(bitRead(analoge_kanaele, 0)) { enable_analog_0 = 1; channels_count++; } else { enable_analog_0 = 0; }
      if(bitRead(analoge_kanaele, 1)) { enable_analog_1 = 1; channels_count++; } else { enable_analog_1 = 0; }
      if(bitRead(analoge_kanaele, 2)) { enable_analog_2 = 1; channels_count++; } else { enable_analog_2 = 0; }
      if(bitRead(analoge_kanaele, 3)) { enable_analog_3 = 1; channels_count++; } else { enable_analog_3 = 0; }
      if(bitRead(analoge_kanaele, 4)) { enable_analog_4 = 1; channels_count++; } else { enable_analog_4 = 0; }
      if(bitRead(analoge_kanaele, 5)) { enable_analog_5 = 1; channels_count++; } else { enable_analog_5 = 0; }
    }    

    // Befehl zum Messintervall verarbeiten
    if(uart_inputstr[0] == 'D') 
    { 
      messintervall_f = atof(tempstr); 
      messintervall = (unsigned long) messintervall_f;
      EEPROM.put(2, messintervall_f);
    }
  }
} // ende uart_auswertung

// Schreibt den Tabellenkopf in die Datei, damit klar ist, welche Spalte zu welchem Sensor gehört
void Tabellenkopfzeile()
{
  String dataString = "Lfd.Nr.,";
  if(enable_analog_5 > 0)
  {
    dataString += "A5, ";
  }
  if(enable_analog_4 > 0)
  {
    dataString += "A4, ";
  }
  if(enable_analog_3 > 0)
  {
    dataString += "A3, ";
  }
  if(enable_analog_2 > 0)
  {
    dataString += "A2, ";
  }
  if(enable_analog_1 > 0)
  {
    dataString += "A1, ";
  }
  if(enable_analog_0 > 0)
  {
    dataString += "A0, ";
  }
  Serial.println(dataString);
  
  File dataFile = SD.open("Sensoren.txt", FILE_WRITE);
    // wenn SD-Card bereit, schreiben
    if (dataFile) 
    {
      dataFile.println(dataString);
      dataFile.close();
    }
}

void setup() {
  // Open serial communications and wait for port to open:
  Serial.begin(9600);
  while (!Serial) {
    ; // wait for serial port to connect. Needed for native USB port only
  }

  Serial.print("Initializing SD card...");

  // see if the card is present and can be initialized:
  if (!SD.begin(chipSelect)) 
  {
    Serial.println("Card failed, or not present");
    // don't do anything more:
    return;
  }
  Serial.println("card initialized.");

  eeprom_lesen();                                   // beim Start EEPROM-Werte auslesen
  Tabellenkopfzeile();
}

void loop() 
{
  if(messung)
  {
    last = millis();                                          // Zeit merken
    messung = false;
        
    // make a string for assembling the data to log:
    String dataString = "";
  
    dataString = String(count++);
    dataString += ",";
      
    // lese Sensoren und hänge Wert an den String an
    if(enable_analog_5 > 0)
    {
      rohwert = analogRead(5);
      dataString += String(rohwert);
      dataString += ",";
    }
    
    if(enable_analog_4 > 0)
    {
      rohwert = analogRead(4);
      dataString += String(rohwert);
      dataString += ",";
    }
    
    if(enable_analog_3 > 0)
    {
      rohwert = analogRead(3);
      dataString += String(rohwert);
      dataString += ",";
    }
    
    if(enable_analog_2 > 0)
    {
      rohwert = analogRead(2);
      dataString += String(rohwert);
      dataString += ",";
    }
    
    if(enable_analog_1 > 0)
    {
      rohwert = analogRead(1);
      dataString += String(rohwert);
      dataString += ",";
    }
    
    if(enable_analog_0 > 0)
    {
      rohwert = analogRead(0);
      dataString += String(rohwert);
      dataString += ",";
    }
  
    // open the file.
    File dataFile = SD.open("sensoren.txt", FILE_WRITE);
  
    // wenn SD-Card bereit, schreiben
    if (dataFile) 
    {
      dataFile.println(dataString);
      dataFile.close();
      // print to the serial port too:
      Serial.println(dataString);
      last = millis();
    }
    // if the file isn't open, pop up an error:
    else 
    {
      Serial.println("error opening datalog.txt");
    }
  }

  // wenn ein Befehl empfangen wurde
  if (complete) 
  {
    //Serial.println(inputstr);
    complete = false;
    uart_auswertung();
  }  
  
  if(millis() >= (last + messintervall)) 
  {
    messung = true;
  }
}


/*
 SerialEvent occurs whenever a new data comes in the
 hardware serial RX.  This routine is run between each
 time loop() runs. Multiple bytes of data may be available.
 */
void serialEvent() 
{
  while (Serial.available()) 
  {
    char uart_inChar = (char)Serial.read();
    if (isAlphaNumeric(uart_inChar) || isPunct(uart_inChar))   //erlaubt: a-z, 0-9, ,. etc.
    {
      uart_inputstr[i++] = uart_inChar;
      if (uart_inChar == ';')                       // Stringende-Kennzeichen
      {
        uart_inputstr[i] = 0;
        complete = true;
        i = 0;
      }
    }
  }
}
